#include <SDL.h>
#include <SDL_gfxPrimitives.h>
#include <SDL_rotozoom.h>

#include "sdl_gfx_gp2x.h"

void DrawLineRGBA(SDL_Surface *dst, int x, int y, int x2, int y2, Uint32 c)
{
	// Draw antialiased line
	aalineColor(dst, x, y, x2, y2, c);
}

void DrawRectangleRGBA(SDL_Surface *dst, int x, int y, int w, int h, Uint32 c)
{
	int x2, y2;

	// Draw rectangle
	x2 = x + w;
	y2 = y + h;
	rectangleColor(dst, x, y, x2, y2, c);
}

void DrawBoxRGBA(SDL_Surface *dst, int x, int y, int w, int h, Uint32 c)
{
	int x2, y2;

	// Calculate x2 and y2
	x2 = x + w;
	y2 = y + h;
	
	// Draw box
	boxColor(dst, x, y, x2, y2, c);
}

void DrawBoxWithBorderRGBA(SDL_Surface *dst, int x, int y, int w, int h, Uint32 c, int s, Uint32 c1)
{
	int x1, y1, x2, y2;
	
	// Draw box
	x1 = x + w;
	y1 = y + h;
	boxColor(dst, x, y, x1, y1, c);

	// Draw left border
	x1 = x - s;
	y1 = y - s;
	x2 = x;
	y2 = y + (h + s);
	boxColor(dst, x1, y1, x2, y2, c1);

	// Draw right border
	x1 = x + w;
	y1 = y - s;
	x2 = x + (w + s);
	y2 = y + (h + s);
	boxColor(dst, x1, y1, x2, y2, c1);

	// Draw top border
	x1 = x;
	y1 = y - s;
	x2 = x + w;
	y2 = y;
	boxColor(dst, x1, y1, x2, y2, c1);

	// Draw bottom border
	x1 = x;
	y1 = y + h;
	x2 = x + w;
	y2 = y + (h + s);
	boxColor(dst, x1, y1, x2, y2, c1);
}

void DrawCircleRGBA(SDL_Surface *dst, int x, int y, int radius, Uint32 c)
{
	// Draw antialiased circle
	aacircleColor(dst, x, y, radius, c);
}

void DrawEllipseRGBA(SDL_Surface *dst, int x, int y, int rx, int ry, Uint32 c)
{

	// Draw antialiased ellipse
	aaellipseColor(dst, x, y, rx, ry, c);
}

void DrawFilledCircleRGBA(SDL_Surface *dst, int x, int y, int radius, Uint32 c)
{
	// Draw filled circle
	filledCircleColor(dst, x, y, radius, c);
}

void DrawFilledEllipseRGBA(SDL_Surface *dst, int x, int y, int rx, int ry, Uint32 c)
{
	// Draw filled ellipse
	filledEllipseColor(dst, x, y, rx, ry, c);
}

void DrawTriangleRGBA(SDL_Surface *dst, int x1, int y1, int x2, int y2, int x3, int y3, Uint32 c)
{
	// Draw antialiased triangle
	aatrigonColor(dst, x1, y1, x2, y2, x3, y3, c);
}

void DrawFilledTrianbleRGBA(SDL_Surface *dst, int x1, int y1, int x2, int y2, int x3, int y3, Uint32 c)
{
	// Draw filled triangle
	filledTrigonColor(dst, x1, y1, x2, y2, x3, y3, c);
}

void DrawRegularTriangleRGBA(SDL_Surface *dst, int x, int y, int d, Uint32 c)
{
	int x1, y1, x2, y2, x3, y3;
	int h;

	// Calculate triangle height
	h = d * cos(30);

	// Calculate coordinates (1 -> bottom-left vertex, 2 -> bottom-right vertex, 3 -> top vertex)
	x1 = x - (d / 2);
	y1 = y + (h / 2);
	x2 = x + (d / 2);
	y2 = y + (h / 2);
	x3 = x;
	y3 = y - (h / 2);

	// Draw antialiased triangle
	aatrigonColor(dst, x1, y1, x2, y2, x3, y3, c);
}

void DrawRegularFilledTriangleRGBA(SDL_Surface *dst, int x, int y, int d, Uint32 c)
{
	int x1, y1, x2, y2, x3, y3;
	int h;

	// Calculate triangle height
	h = sqrt( (d*d) - (d*d/4) );

	// Calculate coordinates (1 -> bottom-left vertex, 2 -> bottom-right vertex, 3 -> top vertex)
	x1 = x - (d / 2);
	y1 = y + (h / 2);
	x2 = x + (d / 2);
	y2 = y + (h / 2);
	x3 = x;
	y3 = y - (h / 2);

	// Draw filled triangle
	filledTrigonColor(dst, x1, y1, x2, y2, x3, y3, c);
}

SDL_Surface *RotateSurface(SDL_Surface *src, double angle, int smooth)
{
	SDL_Surface *surface;

	// Rotate surface
	surface = rotozoomSurface(src, angle, 1, smooth);

	// Return surface
	return surface;
}

SDL_Surface *ZoomSurface(SDL_Surface *src, double zoomx, double zoomy, int smooth)
{
	SDL_Surface *surface;

	// Zoom surface
	surface = zoomSurface(src, zoomx, zoomy, smooth);

	// Return surface
	return surface;
}

SDL_Surface *RotoZoomSurface(SDL_Surface *src, double angle, double zoomx, double zoomy, int smooth)
{
	SDL_Surface *surface;

	// Rotate and zoom surface
	surface = rotozoomSurfaceXY(src, angle, zoomx, zoomy, smooth);

	// Return surface
	return surface;
}
