// DaveC's joystick configuration
// Based on code of miq01

#include <SDL.h>

#include "sdl_joystick_gp2x.h"

const Uint8 SPF = 15;
int jup, jdown, jleft, jright, jupleft, jupright, jdownleft, jdownright;

void KeyDown(Uint8 key)
{
	// Get key pressed and set its variable as true
	switch(key)
	{
		case GP2X_BUTTON_UP: jup = 1; break;
		case GP2X_BUTTON_DOWN: jdown = 1; break;
		case GP2X_BUTTON_LEFT: jleft = 1; break;
		case GP2X_BUTTON_RIGHT: jright = 1; break;
		case GP2X_BUTTON_UPLEFT: jupleft = 1; break;
		case GP2X_BUTTON_UPRIGHT: jupright = 1; break;
		case GP2X_BUTTON_DOWNLEFT: jdownleft = 1; break;
		case GP2X_BUTTON_DOWNRIGHT: jdownright = 1; break;

		case GP2X_BUTTON_START: joystart = 1; break;
		case GP2X_BUTTON_SELECT: joyselect = 1; break;
		case GP2X_BUTTON_A: joya = 1; break;
		case GP2X_BUTTON_B: joyb = 1; break;
		case GP2X_BUTTON_X: joyx = 1; break;
		case GP2X_BUTTON_Y: joyy = 1; break;
		case GP2X_BUTTON_L: joyl = 1; break;
		case GP2X_BUTTON_R: joyr = 1; break;
		case GP2X_BUTTON_VOLUP: joyvolup = 1; break;
		case GP2X_BUTTON_VOLDOWN: joyvoldown = 1; break;
		case GP2X_BUTTON_CLICK: joyclick = 1; break;
		default: break;
	}
}

void KeyUp(Uint8 key)
{
	// Get key released and set its variable as false
	switch(key)
	{
		case GP2X_BUTTON_UP: jup = 0; break;
		case GP2X_BUTTON_DOWN: jdown = 0; break;
		case GP2X_BUTTON_LEFT: jleft = 0; break;
		case GP2X_BUTTON_RIGHT: jright = 0; break;
		case GP2X_BUTTON_UPLEFT: jupleft = 0; break;
		case GP2X_BUTTON_UPRIGHT: jupright = 0; break;
		case GP2X_BUTTON_DOWNLEFT: jdownleft = 0; break;
		case GP2X_BUTTON_DOWNRIGHT: jdownright = 0; break;

		case GP2X_BUTTON_START: joystart = 0; break;
		case GP2X_BUTTON_SELECT: joyselect = 0; break;
		case GP2X_BUTTON_A: joya = 0; break;
		case GP2X_BUTTON_B: joyb = 0; break;
		case GP2X_BUTTON_X: joyx = 0; break;
		case GP2X_BUTTON_Y: joyy = 0; break;
		case GP2X_BUTTON_L: joyl = 0; break;
		case GP2X_BUTTON_R: joyr = 0; break;
		case GP2X_BUTTON_VOLUP: joyvolup = 0; break;
		case GP2X_BUTTON_VOLDOWN: joyvoldown = 0; break;
		case GP2X_BUTTON_CLICK: joyclick = 0; break;
		default: break;
	}
}

void HandleInput()
{
	SDL_Event event;

	// Read events from joystick
	while(SDL_PollEvent(&event))
	{
		switch(event.type)
		{
			case SDL_JOYBUTTONDOWN: KeyDown(event.jbutton.button); break;
			case SDL_JOYBUTTONUP: KeyUp(event.jbutton.button); break;
		}
	}
}

void UpdateInput(int davec)
{
	if(davec == 1)
	{
		// DaveC's case 1
		joy1 = jupleft;
		joy2 = jup;
		joy3 = jupright;
		joy4 = jright;
		joy5 = jdownright;
		joy6 = jdown;
		joy7 = jdownleft;
		joy8 = jleft;
	}

	if(davec == 2)
	{
		// DaveC's case 2
		joy1 = jupleft && !jleft && !jup;
		joy2 = jup || (jup && jupleft) || (jup && jupright);
		joy3 = jupright && !jright && !jup;
		joy4 = jright || (jright && jupright) || (jright && jdownright);
		joy5 = jdownright && !jright && !jdown;
		joy6 = jdown || (jdown && jdownleft) || (jdown && jdownright);
		joy7 = jdownleft && !jleft && !jdown;
		joy8 = jleft || (jleft && jupleft) || (jleft && jdownleft);
	}

	if(davec == 3)
	{
		// DaveC's case 3
		joy1 = jupleft || (jupleft && jup) || (jupleft && jleft);
		joy2 = jup && !jupleft && !jupright;
		joy3 = jupright || (jupright && jup) || (jupright && jright);
		joy4 = jright && !jupright && !jdownright;
		joy5 = jdownright || (jdownright && jright) || (jdownright && jdown);
		joy6 = jdown && !jdownleft && !jdownright;
		joy7 = jdownleft || (jdownleft && jdown) || (jdownleft && jleft);
		joy8 = jleft && !jdownleft && !jupleft;
	}

	if(davec == 4)
	{
		// DaveC's case 4
		joy1 = jupleft && !jup && !jleft;
		joy2 = jup && !jupleft && !jupright;
		joy3 = jupright && !jup && !jright;
		joy4 = jright && !jupright && !jdownright;
		joy5 = jdownright && !jdown && !jright;
		joy6 = jdown && !jdownleft && !jdownright;
		joy7 = jdownleft && !jdown && !jleft;
		joy8 = jleft && !jupleft && !jdownleft;
	}

	if(davec == 5)
	{
		// DaveC's case 5
		joy2 = (jupleft && !jleft) || (jupleft && jup) || jup || (jup && jupright);
		joy4 = (jupright && !jup) || (jupright && jright) || jright || (jright && jdownright);
		joy6 = (jdownright && !jright) || (jdownright && jdown) || jdown || (jdown && jdownleft);
		joy8 = (jdownleft && !jdown) || (jdownleft && jleft) || jleft || (jleft && jupleft);
	}

	if(davec == 6)
	{
		// DaveC's case 6
		joy2 = (jup && !jupleft) || (jup && jupright) || jupright || (jupright && jright);
		joy4 = (jright && !jupright) || (jright && jdownright) || jdownright || (jdownright && jdown);
		joy6 = (jdown && !jdownright) || (jdown && jdownleft) || jdownleft || (jdownleft && jleft);
		joy8 = (jleft && !jdownleft) || (jleft && jupleft) || jupleft || (jupleft && jup);
	}
}

void ReadJoystick(int davec)
{
	if(SDL_GetTicks()-milliseconds > SPF)
	{
		// Read joystick events
		HandleInput();

		// Update variables
		UpdateInput(davec);

		milliseconds = SDL_GetTicks();
	}
}

void PrepareJoystick()
{
	jup=jdown=jleft=jright=jupleft=jupright=jdownleft=jdownright=joystart=joyselect=joya=joyb=joyx=joyy=joyl=joyr=joyvolup=joyvoldown=joyclick = 0;
	joy1=joy2=joy3=joy4=joy5=joy6=joy7=joy8 = 0;
	milliseconds = SDL_GetTicks();
}
