#include <SDL.h>
#include <SDL_ttf.h>

#include "sdl_gp2x.h"
#include "sdl_ttf_gp2x.h"

void InitSDL_TTF()
{
	// Initialize TrueType API
	printf("Initialize TrueType API...");
	if(TTF_Init() < 0)
	{
		printf("\t\t\tERROR\n");
		SDL_Quit();
	}
	else
		printf("\t\t\tOK\n");
}

void QuitSDL_TTF()
{
	// Quit SDL_ttf
	TTF_Quit();
}

TTF_Font *LoadFont(const char *c, int s)
{
	TTF_Font *f;
	
	// Load font	
	printf("Loading \"%s\"...", c);
	f = TTF_OpenFont(c, s);
	if(!f)
	{
		printf("\t\t\t\tERROR\n");
		TTF_Quit();
		SDL_Quit();
	}
	else
		printf("\t\t\t\tOK\n");

	// Return font
	return f;
}

void CloseFont(TTF_Font *f)
{
	// Close font
	printf("Closing font...");
	TTF_CloseFont(f);
	printf("\t\t\t\tOK\n");
}

SDL_Surface *RenderTextSolid(TTF_Font *f, SDL_PixelFormat *fmt, const char *text, Uint32 c)
{
	SDL_Surface *surface;
	Uint8 r, g, b;

	// Get RGB
	SDL_GetRGB(c, fmt, &r, &g, &b);

	SDL_Color fg = {r, g, b};

	// Rendering text into surface
	surface = TTF_RenderText_Solid(f, text, fg);

	// Return surface
	return surface;
}

SDL_Surface *RenderTextShaded(TTF_Font *f, SDL_PixelFormat *fmt, const char *text, Uint32 c, Uint32 bg)
{
	SDL_Surface *surface;
	Uint8 r, g, b;
	Uint8 r1, g1, b1;

	// Get RGB
	SDL_GetRGB(c, fmt, &r, &g, &b);
	SDL_GetRGB(bg, fmt, &r1, &g1, &b1);

	SDL_Color fg = {r, g, b};
	SDL_Color bg1 = {r1, g1, b1};

	// Rendering text into surface
	surface = TTF_RenderText_Shaded(f, text, fg, bg1);

	// Return surface
	return surface;
}

SDL_Surface *RenderTextBlended(TTF_Font *f, SDL_PixelFormat *fmt, const char *text, Uint32 c)
{
	SDL_Surface *surface;
	Uint8 r, g, b;

	// Get RGB
	SDL_GetRGB(c, fmt, &r, &g, &b);

	SDL_Color fg = {r, g, b};

	// Rendering text into surface
	surface = TTF_RenderText_Blended(f, text, fg);

	// Return surface
	return surface;
}

void DrawTextSolid(SDL_Surface *dst, TTF_Font *f, const char *text, int x, int y, Uint32 c)
{
	SDL_Surface *surface;
	SDL_Rect rect;
	int x1, y1, w1, h1;

	// Render text into a surface
	surface = RenderTextSolid(f, dst->format, text, c);

	// Blit surface
	x1 = x-(surface->w / 2);
	y1 = y-(surface->h / 2);
	w1 = surface->w;
	h1 = surface->h;
	rect = (SDL_Rect) {x1, y1, w1, h1};
	SDL_BlitSurface(surface, NULL, dst, &rect);

	// Free surface
	SDL_FreeSurface(surface);
}

void DrawTextShaded(SDL_Surface *dst, TTF_Font *f, const char *text, int x, int y, Uint32 c, Uint32 bg)
{
	SDL_Surface *surface;
	SDL_Rect rect;
	int x1, y1, w1, h1;

	// Render text into a surface
	surface = RenderTextShaded(f, dst->format, text, c, bg);

	// Blit surface
	x1 = x-(surface->w / 2);
	y1 = y-(surface->h / 2);
	w1 = surface->w;
	h1 = surface->h;
	rect = (SDL_Rect) {x1, y1, w1, h1};
	SDL_BlitSurface(surface, NULL, dst, &rect);

	// Free surface
	SDL_FreeSurface(surface);
}

void DrawTextBlended(SDL_Surface *dst, TTF_Font *f, const char *text, int x, int y, Uint32 c)
{
	SDL_Surface *surface;
	SDL_Rect rect;
	int x1, y1, w1, h1;

	// Render text into a surface
	surface = RenderTextBlended(f, dst->format, text, c);

	// Blit surface
	x1 = x-(surface->w / 2);
	y1 = y-(surface->h / 2);
	w1 = surface->w;
	h1 = surface->h;
	rect = (SDL_Rect) {x1, y1, w1, h1};
	SDL_BlitSurface(surface, NULL, dst, &rect);

	// Free surface
	SDL_FreeSurface(surface);
}

void DrawTextWithShadow(SDL_Surface *dst, TTF_Font *f, const char *text, int x, int y, Uint32 c, Uint32 c1)
{
	SDL_Surface *surface, *shadow;
	SDL_Rect rect;
	int x1, y1, w1, h1;

	// Render text and shadow into a surface
	surface = RenderTextBlended(f, dst->format, text, c);
	shadow = RenderTextBlended(f, dst->format, text, c1);

	// Blit surface (shadow)
	x1 = x-(surface->w / 2);
	y1 = y-(surface->h / 2);
	w1 = surface->w;
	h1 = surface->h;
	rect = (SDL_Rect) {x1, y1, w1, h1};
	SDL_BlitSurface(shadow, NULL, dst, &rect);

	// Blit surface
	x1--;
	y1--;
	rect = (SDL_Rect) {x1, y1, w1, h1};
	SDL_BlitSurface(surface, NULL, dst, &rect);

	// Free surface
	SDL_FreeSurface(surface);
	SDL_FreeSurface(shadow);
}
